@@ -12,6 +12,18 @@ from skorecard.reporting.report import BucketTableMethod, SummaryMethod
 from skorecard.features_bucket_mapping import FeaturesBucketMapping
 from skorecard.bucket_mapping import BucketMapping
 from skorecard.reporting import build_bucket_table
+from skorecard.utils.exceptions import NotInstalledError
+from skorecard.utils.validation import is_fitted
+
+# JupyterDash
+try:
+    from jupyter_dash import JupyterDash
+except ModuleNotFoundError:
+    JupyterDash = NotInstalledError("jupyter-dash", "dashboard")
+
+
+from skorecard.apps.app_layout import add_basic_layout
+from skorecard.apps.app_callbacks import add_bucketing_callbacks
 
 PathLike = TypeVar("PathLike", str, pathlib.Path)
 
@@ -27,10 +39,47 @@ class BaseBucketer(BaseEstimator, TransformerMixin, PlotBucketMethod, BucketTabl
             raise TypeError("The data set should be a pandas dataframe")
         return X.copy()
 
+    @staticmethod
+    def _check_y(y):
+        # checks that y is an appropriate type and shape
+        if y is None:
+            return y
+        y = y.copy()
+        if isinstance(y, pd.DataFrame):
+            if y.shape[1] != 1:
+                raise AttributeError("If passing y as a DataFrame, it must be 1 column.")
+            y = y.values.reshape(
+                -1,
+            )
+
+        elif isinstance(y, pd.core.series.Series):
+            y = y.values
+
+        elif isinstance(y, np.ndarray):
+            if y.ndim > 2:
+                raise AttributeError("If passing y as a Numpy array, y must be a 1-dimensional")
+            elif y.ndim == 2:
+                if y.shape[1] != 1:
+                    raise AttributeError("If passing y as a Numpy array, y must be a 1-dimensional")
+                else:
+                    y = y.reshape(
+                        -1,
+                    )
+        else:
+            raise TypeError("y must be either a Pandas column or a Numpy array")
+        return y
+
     @staticmethod
     def _is_allowed_missing_treatment(missing_treatment):
         # checks if the argument for missing_values is valid
-        allowed_str_missing = ["separate", "most_frequent", "most_risky", "least_risky", "neutral", "similar"]
+        allowed_str_missing = [
+            "separate",
+            "most_frequent",
+            "most_risky",
+            "least_risky",
+            "neutral",
+            "similar",
+        ]
 
         if type(missing_treatment) == str:
             if missing_treatment not in allowed_str_missing:
@@ -161,13 +210,14 @@ class BaseBucketer(BaseEstimator, TransformerMixin, PlotBucketMethod, BucketTabl
     def fit(self, X, y=None):
         """Fit X, y."""
         X = self._is_dataframe(X)
+        y = self._check_y(y)
         self.variables = self._check_variables(X, self.variables)
         self._verify_specials_variables(self.specials, X.columns)
 
         if isinstance(y, pd.Series):
             y = y.values
 
-        features_bucket_mapping_ = {}
+        self.features_bucket_mapping_ = FeaturesBucketMapping()
         self.bucket_tables_ = {}
 
         for feature in self.variables:
@@ -188,20 +238,58 @@ class BaseBucketer(BaseEstimator, TransformerMixin, PlotBucketMethod, BucketTabl
             assert isinstance(X_flt, pd.Series)
             splits, right = self._get_feature_splits(feature, X=X_flt, y=y_flt, X_unfiltered=X)
 
-            # Deal with missing values
-            if self.missing_treatment in [
-                "separate",
-                "most_frequent",
-                "most_risky",
-                "least_risky",
-                "neutral",
-                "similar",
-            ]:
-                missing_bucket = None
-            if isinstance(self.missing_treatment, dict):
-                missing_bucket = self.missing_treatment.get(feature)
+            self._update_column_fit(X, y, feature, special, splits, right)
 
-            features_bucket_mapping_[feature] = BucketMapping(
+        self._generate_summary(X, y)
+
+        return self
+
+    def _update_column_fit(self, X, y, feature, special, splits, right, generate_summary=False):
+        """
+        Extract out part of the fit for a column.
+
+        Useful when we want to interactively update the fit.
+        """
+        # Deal with missing values
+        if self.missing_treatment in [
+            "separate",
+            "most_frequent",
+            "most_risky",
+            "least_risky",
+            "neutral",
+            "similar",
+        ]:
+            missing_bucket = None
+        if isinstance(self.missing_treatment, dict):
+            missing_bucket = self.missing_treatment.get(feature)
+
+        self.features_bucket_mapping_[feature] = BucketMapping(
+            feature_name=feature,
+            type=self.variables_type,
+            map=splits,
+            right=right,
+            specials=special,
+            missing_bucket=missing_bucket,
+        )
+
+        # Calculate the bucket table
+        self.bucket_tables_[feature] = build_bucket_table(
+            X,
+            y,
+            column=feature,
+            bucket_mapping=self.features_bucket_mapping_.get(feature),
+        )
+
+        if self.missing_treatment in [
+            "most_frequent",
+            "most_risky",
+            "least_risky",
+            "neutral",
+            "similar",
+        ]:
+            missing_bucket = self._find_missing_bucket(feature=feature)
+            # Repeat above procedure now we know the bucket distribution
+            self.features_bucket_mapping_[feature] = BucketMapping(
                 feature_name=feature,
                 type=self.variables_type,
                 map=splits,
@@ -210,39 +298,41 @@ class BaseBucketer(BaseEstimator, TransformerMixin, PlotBucketMethod, BucketTabl
                 missing_bucket=missing_bucket,
             )
 
-            # Calculate the bucket table
+            # Recalculate the bucket table with the new bucket for missings
             self.bucket_tables_[feature] = build_bucket_table(
                 X,
                 y,
                 column=feature,
-                bucket_mapping=features_bucket_mapping_.get(feature),
+                bucket_mapping=self.features_bucket_mapping_.get(feature),
             )
 
-            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky", "neutral", "similar"]:
-                missing_bucket = self._find_missing_bucket(feature=feature)
-                # Repeat above procedure now we know the bucket distribution
-                features_bucket_mapping_[feature] = BucketMapping(
-                    feature_name=feature,
-                    type=self.variables_type,
-                    map=splits,
-                    right=right,
-                    specials=special,
-                    missing_bucket=missing_bucket,
-                )
+        if generate_summary:
+            self._generate_summary(X, y)
 
-                # Recalculate the bucket table with the new bucket for missings
-                self.bucket_tables_[feature] = build_bucket_table(
-                    X,
-                    y,
-                    column=feature,
-                    bucket_mapping=features_bucket_mapping_.get(feature),
-                )
+    def fit_interactive(self, X, y=None, mode="external", **server_kwargs):
+        """
+        Fit a bucketer and then interactive edit the fit using a dash app.
 
-        self.features_bucket_mapping_ = FeaturesBucketMapping(features_bucket_mapping_)
+        Note we are using a [jupyterdash](https://medium.com/plotly/introducing-jupyterdash-811f1f57c02e) app,
+        which supports 3 different modes:
 
-        self._generate_summary(X, y)
+        - 'external' (default): Start dash server and print URL
+        - 'inline': Start dash app inside an Iframe in the jupyter notebook
+        - 'jupyterlab': Start dash app as a new tab inside jupyterlab
 
-        return self
+        """
+        # We need to make sure we only fit if not already fitted
+        # This prevents a user loosing manually defined boundaries
+        # when re-running .fit_interactive()
+        if not is_fitted(self):
+            self.fit(X, y)
+
+        import dash_bootstrap_components as dbc
+
+        self.app = JupyterDash(__name__, external_stylesheets=[dbc.themes.BOOTSTRAP])
+        add_basic_layout(self)
+        add_bucketing_callbacks(self, X, y)
+        self.app.run_server(mode=mode, **server_kwargs)
 
     def transform(self, X: pd.DataFrame, y=None) -> pd.DataFrame:
         """Transforms an array into the corresponding buckets fitted by the Transformer.
@@ -256,6 +346,7 @@ class BaseBucketer(BaseEstimator, TransformerMixin, PlotBucketMethod, BucketTabl
         """
         check_is_fitted(self)
         X = self._is_dataframe(X)
+        y = self._check_y(y)
 
         for feature in self.variables:
             bucket_mapping = self.features_bucket_mapping_.get(feature)
